<?php
/**
 * The Route
 **/

$app->notFound(function () use ($app) {

  $path = $app->request()->getResourceUri();;

  $content_root = Statamic::get_content_root();
  $content_type = Statamic::get_content_type();

  $template_list = array('default');
  $app->config['current_path'] = $path;

  if (file_exists("{$content_root}/{$path}.{$content_type}") || is_dir("{$content_root}/{$path}")) {
    // endpoint or folder exists!
  } else {
    $path = Statamic_Helper::resolve_path($path);
    $app->config['current_path'] = $path; # override global current_path
  }

  if (file_exists("{$content_root}/{$path}.{$content_type}")) {
    // @todo: Load Post if a date/numerical entry, else page
    $template_list[] = 'post';
    $page     = basename($path);
    $folder   = substr($path, 0, (-1*strlen($page))-1);

    $data = Statamic::get_content_meta($page, $folder);

    $visible = true;
    if (isset($data['status'])) {
      if ($data['status'] != 'live' && $data['status'] != 'hidden') {
        $data = Statamic::get_content_meta("404", "/");
        $template_list = array('404');
        $visible = false;
      }
    }

    if ($visible) {
      $prev = Statamic::find_prev($page, $folder);
      $next = Statamic::find_next($page, $folder);

      if ($prev) {
        $prev = Statamic_Helper::remove_numerics_from_path($folder."/".$prev);
        $prev = Statamic_Helper::reduce_double_slashes($prev);
      }

      if ($next) {
        $next = Statamic_Helper::remove_numerics_from_path($folder."/".$next);
        $next = Statamic_Helper::reduce_double_slashes($next);
      }

      $data['prev'] = null;
      $data['next'] = null;

      $folder_data = Statamic::get_content_meta("page", dirname($path));
      if ( ! isset($data['_template']) && isset($folder_data['_default_folder_template'])) {
        $data['_template'] = $folder_data['_default_folder_template'];
      }
    }

  } else if (Statamic::is_taxonomy_url($path)) {
    list($type, $slug) = Statamic::get_taxonomy_criteria($path);
    $data = Statamic::get_content_meta($type, Statamic::remove_taxonomy_from_path($path, $type, $slug));
    $data['taxonomy_slug'] = urldecode($slug);
    $template_list[] = $type;
    $template_list[] = "taxonomies";

  } else if (is_dir("{$content_root}/{$path}")) {
    $data = Statamic::get_content_meta("page", $path);
    //$data['entries'] = Statamic::get_content_list($path);
  } else {
    $data = Statamic::get_content_meta("404", "/");
    $template_list = array('404');
  }

  # Set template
  if (isset($data['_template'])){
    $template_list[] = $data['_template'];
  }

  # Set layout
  if (isset($data['_layout'])) {
    Statamic_View::set_layout("layouts/{$data['_layout']}");
  }

  Statamic_View::set_templates(array_reverse($template_list));

  # Set type
  if (isset($data['_type'])) {
    if ($data['_type'] == 'rss') {
      $data['_xml_header'] = '<?xml version="1.0" encoding="utf-8"?>';
      header("Content-Type: application/xml; charset=ISO-8859-1"); 
    }
  }
  $app->render(null, $data);
  exit();
});