<?php

/**
 * The Routes
 **/

function authenticateForRole($role = 'member') {
    $aa = Slim::getInstance();
    $user = Statamic_Auth::get_current_user();
    if ($user) {
      if ($user->has_role($role) === false) {
        $aa->redirect($aa->urlFor('denied'));
      }
    } else {
      $aa->redirect($aa->urlFor('login'));
    } 
    return true;
} 

function doStatamicVersionCheck($app) {
  $cookie = $app->getEncryptedCookie('stat_latest_version');
  if (!$cookie) {
    $license = Statamic::get_license_key();
    $site_url = Statamic::get_site_url();
    $parts = parse_url($site_url);
    $domain = isset($parts['host']) ? $parts['host'] : '/';

    $url = "http://outpost.statamic.com/check?v=".urlencode(STATAMIC_VERSION)."&l=".urlencode($license)."&d=".urlencode($domain);
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_URL, $url); 
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_TIMEOUT, '3');
    $content = trim(curl_exec($ch));
    curl_close($ch);

    if ($content <> '') {
      $response = json_decode($content);
      if ($response && $response->status == 'ok') {
        $app->setEncryptedCookie('stat_latest_version', $response->current_version);
        $app->setEncryptedCookie('stat_latest_version_url', $response->url);
        $app->config['latest_version_url'] = $response->current_version;
        $app->config['latest_version'] = $response->current_version;
      } else {
        $app->config['latest_version_url'] = '';
        $app->config['latest_version'] = '';
      }
    }
  } else {
    $app->config['latest_version'] = $cookie;
    $app->config['latest_version_url'] = $app->getEncryptedCookie('stat_latest_version_url');
  }
}

$admin_app->get('/',  function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);

  if (isset($admin_app->config['dashboard_redirects_to']) && $admin_app->config['dashboard_redirects_to'] !== FALSE) {
    $location = $admin_app->config['dashboard_redirects_to'];
    $admin_app->redirect($location);
  }

  $template_list = array("dashboard");
  Statamic_View::set_templates(array_reverse($template_list));
  $admin_app->render(null, array('route' => 'root', 'app' => $admin_app));
})->name('home');


// AUTH RELATED FUNCTION
// --------------------------------------------------------
$admin_app->get('/denied', function() use ($admin_app) {
  $template_list = array("denied");
  Statamic_View::set_templates(array_reverse($template_list));
  Statamic_View::set_layout("layouts/login");
  $admin_app->render(null, array('route' => 'login', 'app' => $admin_app));
})->name('denied');



$admin_app->get('/login', function() use ($admin_app) {
  $template_list = array("login");
  Statamic_View::set_templates(array_reverse($template_list));
  Statamic_View::set_layout("layouts/login");
  $admin_app->render(null, array('route' => 'login', 'app' => $admin_app));
})->name('login');



$admin_app->get('/logout', function() use ($admin_app) {
  Statamic_Auth::logout();
  $admin_app->redirect($admin_app->urlFor('home'));
})->name('logout');



$admin_app->post('/login', function() use ($admin_app) {
  $app = Slim::getInstance();

  $login = $app->request()->post('login');
  $username = $login['username'];
  $password = $login['password'];

  $errors = array();
  // Auth login
  // if success direct to admin homepage
  if (Statamic_Auth::login($username, $password)) {
    $app->redirect($app->urlFor('home'));
  } else {
    $errors = array('login' => 'Incorrect username or password. Try again.');
  }
  // else show login failed page
    // add error message

  $template_list = array("login");
  Statamic_View::set_templates(array_reverse($template_list));
  Statamic_View::set_layout("layouts/login");
  $admin_app->render(null, array('route' => 'login', 'app' => $admin_app, 'errors' => $errors));
})->name('login-submit');



// ERROR FUNCTION
// --------------------------------------------------------
$admin_app->get('/error', function() use ($admin_app) {
  $template_list = array("error");
  Statamic_View::set_templates(array_reverse($template_list));
  Statamic_View::set_layout("layouts/login");
  $admin_app->render(null, array('route' => 'login', 'app' => $admin_app));
})->name('error');



// PUBLICATION
// --------------------------------------------------------
$admin_app->get('/pages', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);
  $template_list = array("pages");

  if (!Statamic::is_content_writable()) {
    print "HERE";
    $admin_app->flash('error', 'Content folder not writable');
    $url = $admin_app->urlFor('error')."?code=write_permission";
    $admin_app->redirect($url);
    return;
  }

  $path = "";
  $path = $admin_app->request()->get('path');
  $errors = array();
  $pages = Statamic::get_content_tree('/', 1, 1000, true, false);

  $node['type'] = 'home';
  $node['url'] = "/page";
  $node['slug'] = "/";

  $meta = Statamic::get_content_meta("page", "");
  //$node['meta'] = $meta;

  if (isset($meta['title'])) $node['title'] = $meta['title'];
  $node['depth'] = 1;
  
  array_unshift($pages, $node);

  Statamic_View::set_templates(array_reverse($template_list));
  $admin_app->render(null, array('route' => 'pages', 'app' => $admin_app
    , 'errors' => $errors
    , 'path' => $path
    , 'pages' => $pages
    ));
})->name('pages');



$admin_app->get('/entries', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);
  $content_root = Statamic::get_content_root();
  $template_list = array("entries");

  $path = "";
  $path = $admin_app->request()->get('path');
  $errors = array();

  $path = $admin_app->request()->get('path');
  if ($path) {
    $type = "none";
    if (file_exists("{$content_root}/{$path}/fields.yaml")) { 
      $fields_raw = file_get_contents("{$content_root}/{$path}/fields.yaml");
      $fields_data = Spyc::YAMLLoad($fields_raw);

      if (isset($fields_data['type']['prefix'])) {
        $type = $fields_data['type']['prefix'];
      }
    } else {
    }

    $entries = Statamic::get_content_list($path,null,0,true,true,'date', 'desc', null, null, true);
    Statamic_View::set_templates(array_reverse($template_list));
    $admin_app->render(null, array('route' => 'pages', 'app' => $admin_app
      , 'errors' => $errors
      , 'path' => $path
      , 'entries' => $entries
      , 'type' => $type
      ));
  }
})->name('entries');


// LOGIC
// - VALIDATE
// - SAVE TO ORIGINAL FILENAME
// - IF NECESSARY: RENAME



// POST: PUBLISH
$admin_app->post('/publish', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);

  $content_root = Statamic::get_content_root();
  $content_type = Statamic::get_content_type();

  $app = Slim::getInstance();
  $path = $app->request()->get('path');

  if ($path) {
    $index_file = false;

    // 1. Validate
    $form_data = $app->request()->post('page');
    if ($form_data) {

      if (Statamic_Helper::ends_with($path, 'page')) {
        $index_file = true;
      }

      $errors = array();
      if ($index_file) {
        // some different validation rules
        $slug = $form_data['meta']['slug'];
        if ($slug == '') {
          $errors = array('slug' => 'is required');
        } else {
          if ($slug != $form_data['original_slug']) {


            $file = $content_root."/".dirname($path)."/page.".$content_type;
            $check_file = str_replace($form_data['original_slug'], $slug, $file);
            if (file_exists($check_file)) {
              $errors = array('slug' => 'already exists!');
            }
          }
        }
      } else {
        if (isset($form_data['new'])) {
          $entries = Statamic::get_content_list($path,null,0,true,true);
        } else {
          $entries = Statamic::get_content_list(dirname($path),null,0,true,true);
        }

        $slug = $form_data['meta']['slug'];
        if ($slug == '') {
          $errors = array('slug' => 'is required');
        } else {
          // do we have this slug already?
          if (isset($form_data['new']) || $slug != $form_data['original_slug']) {
            if (Statamic_Validate::content_slug_exists($entries, $slug)) {
              $errors = array('slug' => 'is already used');
            }
          }
        }

        // generate slug & datestamp/number
        $datestamp = '';
        $numeric = '';
        if ($form_data['type'] == 'date') {
          // STANDARDIZE INPUT
          $datestamp = $form_data['meta']['publish-date'];
          if ($datestamp == '') {
            $errors = array('datestamp' => 'is required');
          }
        } else if ($form_data['type'] == 'number') {
          $numeric = $form_data['meta']['publish-numeric'];
          if ($numeric == '') {
            $errors = array('numeric' => 'is required');
          }
        }
      }

      if (sizeof($errors) > 0) {
        // REPOPULATE IF THERE IS AN ERROR
        if (isset($form_data['new'])) {
          $data['new'] = $form_data['new'];
        }
        $data['path'] = $path;
        $data['page'] = '';
        $data['title'] = $form_data['yaml']['title'];
        $folder = $form_data['folder'];
        $data['folder'] = $form_data['folder'];
        $data['content'] = $form_data['content'];
        $data['content_raw'] = $form_data['content'];
        $data['type'] = $form_data['type'];
        $data['errors'] = $errors;

        $data['slug'] = $form_data['meta']['slug' ];

        $data['full_slug'] = $form_data['full_slug'];
        $data['original_slug'] = $form_data['original_slug'];
        $data['original_datestamp'] = $form_data['original_datestamp'];
        $data['original_numeric'] = $form_data['original_numeric'];

        if (!$index_file) {
          $data['datestamp'] = strtotime($datestamp);
          $data['numeric'] = $numeric;
        }

        $data['_template'] = $form_data['yaml']['_template'];

        $data['templates'] = Statamic::get_templates_list();
        $data['layouts'] = Statamic::get_layouts_list();

        $fields_data = null;
        $content_root = Statamic::get_content_root();

        if ($data['type'] != 'none' && file_exists("{$content_root}/{$folder}/fields.yaml")) { 
          $fields_raw = file_get_contents("{$content_root}/{$folder}/fields.yaml");
          $fields_data = Spyc::YAMLLoad($fields_raw);
        }
        if ($fields_data) { 
          $data['fields'] = $fields_data['fields'];
          $data['type'] = $fields_data['type']['prefix'];

          if ($data['type'] == 'date') {
          } else if ($data['type'] == 'number') {
          }
        }

        $template_list = array("publish");
        Statamic_View::set_templates(array_reverse($template_list));
        $admin_app->render(null, array('route' => 'publish', 'app' => $admin_app)+$data);
        return;
      }
    } else {
      print "no form data";
    }
  } else {
    print "no form data";
  }

  // if we got here, have no errors
  // save to original file if not new
  if (isset($form_data['new'])) {
    if ($form_data['type'] == 'date') {
      $file = $content_root."/".$path."/".$datestamp."-".$slug.".".$content_type;
    } else if ($form_data['type'] == 'number') {
      $file = $content_root."/".$path."/".$numeric.".".$slug.".".$content_type;
    } else {
      $file = $content_root."/".$path."/".$form_data['original_slug'].".".$content_type;
    }
    $folder = $path;
  } else {
    $folder = dirname($path);
    if ($form_data['type'] == 'date') {
      $file = $content_root."/".dirname($path)."/".$form_data['original_datestamp']."-".$form_data['original_slug'].".".$content_type;
    } else if ($form_data['type'] == 'number') {
      $file = $content_root."/".dirname($path)."/".$form_data['original_numeric'].".".$form_data['original_slug'].".".$content_type;
    } else {
      if ($index_file) {
        $file = $content_root."/".dirname($path)."/page.".$content_type;
      } else {
        $file = $content_root."/".dirname($path)."/".$form_data['original_slug'].".".$content_type;
      }
    }
  }

  // load the original yaml
  if (isset($form_data['new'])) {
    $file_data = array();
  } else {
    $page = basename($path);
    $folder = dirname($path);
    print $page;
    print $folder;
    $file_data = Statamic::get_content_meta($page, $folder, true);
  }

  # Post-processing for Fieldtypes api
  
  if (file_exists("{$content_root}/{$folder}/fields.yaml")) { 
      $fields_raw = file_get_contents("{$content_root}/{$folder}/fields.yaml");
      $fields_data = Spyc::YAMLLoad($fields_raw);
      $field_settings = $fields_data['fields'];
  }
  foreach ($form_data['yaml'] as $field => $value) {

    if (isset($field_settings[$field]['type'])) {
      $value = Fieldtype::process_field_data($field_settings[$field]['type'], $value);
    }
    if (is_array($value)) {
      $file_data[$field] = $value;
    } else {
      if ($value <> '') {
        $file_data[$field] = $value;
      }
    }
  }

  unset($file_data['content']);
  unset($file_data['content_raw']);

  $file_content = Statamic_Helper::build_file_content($file_data, $form_data['content']);

  // don't save! testing
  file_put_contents($file, $file_content);

  // ### SEE IF WE NEED TO RENAME THE FILE
  if (isset($form_data['new'])) {
  } else {
    // rename the file
    $new_slug = $form_data['meta']['slug'];

    if ($form_data['type'] == 'date') {
      $new_datestamp = $form_data['meta']['publish-date'];
      $new_file = $content_root."/".dirname($path)."/".$new_datestamp."-".$new_slug.".".$content_type;
    } else if ($form_data['type'] == 'number') {
      $new_numeric = $form_data['meta']['publish-numeric'];
      $new_file = $content_root."/".dirname($path)."/".$new_numeric.".".$new_slug.".".$content_type;
    } else {
      if ($index_file) {
        $new_file = str_replace($form_data['original_slug'], $new_slug, $file);
      } else {
        $new_file = $content_root."/".dirname($path)."/".$new_slug.".".$content_type;
      }
    }
    if ($file != $new_file) {
      if ($index_file) {
        rename(dirname($file), dirname($new_file));
      } else {
        rename($file, $new_file);
      }
    }
  }

  // rediect back to entries
  if ($form_data['type'] == 'none') {
    $app->flash('success', 'Page saved successfully!');
    $url = $app->urlFor('pages')."?path=".$folder;
    $app->redirect($url);
  } else {
    $app->flash('success', 'Entry saved successfully!');
    $url = $app->urlFor('entries')."?path=".$folder;
    $app->redirect($url);
  }

}); 

// GET: DELETE ENTRY
$admin_app->get('/deletentry', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);
  $content_root = Statamic::get_content_root();
  $content_type = Statamic::get_content_type();

  $path = $admin_app->request()->get('path');
  $folder = dirname($path);
  $file = $content_root."/".$path.".".$content_type;
  if (file_exists($file)) {
    // rediect back to entries
    unlink($file);
    $admin_app->flash('success', 'Entry successfully deleted!');
    $url = $admin_app->urlFor('entries')."?path=".$folder;
    $admin_app->redirect($url);
  }
})->name('deleteentry');

// GET: PUBLISH
$admin_app->get('/publish', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);
  $content_root = Statamic::get_content_root();
  $app = Slim::getInstance();

  $data = array();
  $path = $app->request()->get('path');
  $new = $app->request()->get('new');
  if ($path) {

    if ($new) {
      $data['new'] = 'true';
      $page     = 'new-slug';
      $folder   = $path;
      $data['full_slug'] = dirname($path);
      $data['slug'] = '';
      $data['path'] = $path;
      $data['page'] = '';
      $data['title'] = '';
      $data['folder'] = $folder;
      $data['content'] = '';
      $data['content_raw'] = '';

      $data['datestamp'] = time();

      $data['original_slug'] = '';
      $data['original_datestamp'] = '';
      $data['original_numeric'] = '';

    } else {
      $page     = basename($path);
      $folder   = substr($path, 0, (-1*strlen($page))-1);

      if (!Statamic::content_exists($page, $folder)) {
        $app->flash('error', 'Content not found!');
        $url = $app->urlFor('pages');
        $app->redirect($url);
        return;
      }

      $data = Statamic::get_content_meta($page, $folder, true);

      $data['title'] = isset($data['title']) ? $data['title'] : '';
      $data['slug'] = basename($path);
      $data['full_slug'] = $folder."/".$page;
      $data['path'] = $path;
      $data['folder'] = $folder;
      $data['page'] = $page;
      $data['type'] = 'none';
      $data['original_slug'] = '';
      $data['original_datestamp'] = '';
      $data['original_numeric'] = '';
      $data['datestamp'] = 0;

      if ($page == 'page') {
        $page = $folder;
        $folder = dirname($folder);
        $data['full_slug'] = $page;
      }
    }

    $content_root = $content_root;
    if ($data['slug'] != 'page' && file_exists("{$content_root}/{$folder}/fields.yaml")) { 
      $fields_raw = file_get_contents("{$content_root}/{$folder}/fields.yaml");
      $fields_data = Spyc::YAMLLoad($fields_raw);
      $data['fields'] = $fields_data['fields'];
      $data['type'] = $fields_data['type']['prefix'];

      if ($data['type'] == 'date') {
        $data['full_slug'] = $folder;
        $data['original_slug'] = substr($page, 11);
        $data['slug'] = substr($page, 11);
        $data['original_datestamp'] = substr($page, 0, 10);
        if (!$new) {
          $data['datestamp'] = strtotime(substr($page, 0, 10));
          $data['full_slug'] = $folder."/".$data['original_slug'];
        }
      } else if ($data['type'] == 'number') {
        if ($new) {
          $data['original_numeric'] = Statamic::get_next_numeric($folder);
          $data['numeric'] = Statamic::get_next_numeric($folder);
          $data['full_slug'] = $folder;
        } else {
          $numeric = Statamic_Helper::get_numeric($page);
          $data['slug'] = substr($page, strlen($numeric)+1);
          $data['original_slug'] = substr($page, strlen($numeric)+1);
          $data['numeric'] = $numeric;
          $data['original_numeric'] = $numeric;
          $data['full_slug'] = $folder."/".$data['original_slug'];
        }
      }
    } else {
      $data['slug'] = $page;
      $data['original_slug'] = $page;
    }


  } else {
    print "NO PATH";
  }

  $data['templates'] = Statamic::get_templates_list();
  $data['layouts'] = Statamic::get_layouts_list();

  $template_list = array("publish");
  Statamic_View::set_templates(array_reverse($template_list));
  $admin_app->render(null, array('route' => 'publish', 'app' => $admin_app)+$data);
})->name('publish');


// MEMBERS
// --------------------------------------------------------
$admin_app->get('/members', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);

  $members = Statamic_Auth::get_user_list();
  $data['members'] = $members;

  $template_list = array("members");
  Statamic_View::set_templates(array_reverse($template_list));
  $admin_app->render(null, array('route' => 'members', 'app' => $admin_app)+$data);
})->name('members');


// POST: MEMBER
// --------------------------------------------------------
$admin_app->post('/member', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);

  $data = array();
  $name = $admin_app->request()->get('name');

  $form_data = $admin_app->request()->post('member');
  if ($form_data) {
    $errors = array();
    // VALIDATE
    if (isset($form_data['new'])) {
      $name = $form_data['name'];
      if ($name == '') {
        $errors['name'] = 'is required';
      } else {
        if (Statamic_Auth::user_exists($name)) {
          $errors['name'] = 'is already taken';
        }
      }
      if ((!isset($form_data['yaml']['password'])) || (!isset($form_data['yaml']['password']))) {
        $errors['password'] = 'and confirmation is required';
      } else {
        if ($form_data['yaml']['password'] == '') {
          $errors['password'] = 'must be at least 1 character';
        } else if ($form_data['yaml']['password'] != $form_data['yaml']['password_confirmation']) {
          $errors['password'] = 'and confirmation do not match';
        }
      }
    } else {
      if ($form_data['name'] <> $form_data['original_name']) {
        if (Statamic_Auth::user_exists($form_data['name'])) {
          $errors['name'] = 'is already taken';
        }
      }

      if (isset($form_data['yaml']['password'])) {
        if ((!isset($form_data['yaml']['password'])) || (!isset($form_data['yaml']['password']))) {
          $errors['password'] = 'and confirmation is required';
        } else {
          if ($form_data['yaml']['password'] <> '') {
            if ($form_data['yaml']['password'] != $form_data['yaml']['password_confirmation']) {
              $errors['password'] = 'and confirmation do not match';
            }
          }
        }
      }
    }

    if (sizeof($errors) > 0) {
      // repopulate and re-render
      $data['errors'] = $errors;

      $data['name'] = $form_data['name'];
      $data['first_name'] = $form_data['yaml']['first_name'];
      $data['last_name'] = $form_data['yaml']['last_name'];
      $data['roles'] = $form_data['yaml']['roles'];
      $data['biography'] =  $form_data['biography'];
      $data['original_name'] = $form_data['original_name'];

      $template_list = array("member");
      Statamic_View::set_templates(array_reverse($template_list));
      $admin_app->render(null, array('route' => 'publish', 'app' => $admin_app)+$data);
      return;
    }

    // IF NOT ERRORS SAVE
    if (isset($form_data['new'])) {
      $user = new Statamic_User(array());
      $user->set_name($name);
    } else {
      $user = Statamic_User::load($name);
    }

    $user->set_first_name($form_data['yaml']['first_name']);
    $user->set_last_name($form_data['yaml']['last_name']);
    $user->set_roles($form_data['yaml']['roles']);
    $user->set_biography_raw($form_data['biography']);

    $encrypted = false;
    if (isset($form_data['yaml']['password_encrypted']) && $form_data['yaml']['password_encrypted']) {
      $encrypted = true;
    }
    if (isset($form_data['yaml']['password']) && $form_data['yaml']['password'] <> '') {
      $user->set_password($form_data['yaml']['password'], $encrypted);
    } elseif ($encrypted) { // encrypt a password left blank
      $user->set_password($user->get_password(), $encrypted);
    }

    $user->save();

    // Rename?
    if (!isset($form_data['new'])) {
      $user->rename($form_data['name']);
    }

    // REDIRECT
    $admin_app->flash('success', 'Member successfully saved!');
    $url = $admin_app->urlFor('members');
    $admin_app->redirect($url);
  }
});


// GET: MEMBER
// --------------------------------------------------------
$admin_app->get('/member', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);
  $data = array();

  $name = $admin_app->request()->get('name');
  $new = $admin_app->request()->get('new');

  if ($new) {
    $data['name']           = '';
    $data['new']            = 'true';
    $data['content_raw']    = '';
    $data['original_name']  = '';

    $data['first_name']     = '';
    $data['last_name']      = '';
    $data['roles']          = '';

    $data['biography']      = '';

  } else {
    $user = Statamic_Auth::get_user($name);
    if (!$user) {
      die("Error");
    }
    $data['name'] = $name;
    $data['first_name'] = $user->get_first_name();
    $data['last_name'] = $user->get_last_name();
    $data['roles'] = $user->get_roles_list();

    $data['biography'] =  $user->get_biography_raw();

    $data['original_name'] = $name;
  }

  $template_list = array("member");
  Statamic_View::set_templates(array_reverse($template_list));
  $admin_app->render(null, array('route' => 'members', 'app' => $admin_app)+$data);
})->name('member');


// GET: DELETE MEMBER
$admin_app->get('/deletemember', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);

  $name = $admin_app->request()->get('name');
  if (Statamic_Auth::user_exists($name)) {
    $user = Statamic_Auth::get_user($name);
    $user->delete();
  }

  // REDIRECT
  $admin_app->flash('info', 'Member deleted');
  $url = $admin_app->urlFor('members');
  $admin_app->redirect($url);
})->name('deletemember');


// Account
// --------------------------------------------------------
$admin_app->get('/account', function() use ($admin_app) {
  authenticateForRole('admin');
  doStatamicVersionCheck($admin_app);

  $template_list = array("account");
  Statamic_View::set_templates(array_reverse($template_list));
  $admin_app->render(null, array('route' => 'members', 'app' => $admin_app));
})->name('account');